@@ -53,7 +53,7 @@ struct _GdkGCPrivate
 
 #define GDK_GC_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDK_TYPE_GC, GdkGCPrivate))
 
-G_DEFINE_TYPE (GdkGC, gdk_gc, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GdkGC, gdk_gc, G_TYPE_OBJECT)
 
 static void
 gdk_gc_class_init (GdkGCClass *class)
