@@ -39,10 +39,9 @@
 #include "gtkintl.h"
 #include "gtkalias.h"
 
-#define CHILD_SPACING     1
-
 static const GtkBorder default_default_border = { 1, 1, 1, 1 };
 static const GtkBorder default_default_outside_border = { 0, 0, 0, 0 };
+static const GtkBorder default_inner_border = { 1, 1, 1, 1 };
 
 /* Time out before giving up on getting a key release when animating
  * the close button.
@@ -85,8 +84,6 @@ struct _GtkButtonPrivate
   guint32      grab_time;
 };
 
-static void gtk_button_class_init     (GtkButtonClass     *klass);
-static void gtk_button_init           (GtkButton          *button);
 static void gtk_button_destroy        (GtkObject          *object);
 static void gtk_button_set_property   (GObject            *object,
                                        guint               prop_id,
@@ -140,37 +137,9 @@ static void gtk_button_grab_notify     (GtkWidget             *widget,
 					gboolean               was_grabbed);
 
 
-
-static GtkBinClass *parent_class = NULL;
 static guint button_signals[LAST_SIGNAL] = { 0 };
 
-
-GType
-gtk_button_get_type (void)
-{
-  static GType button_type = 0;
-
-  if (!button_type)
-    {
-      static const GTypeInfo button_info =
-      {
-	sizeof (GtkButtonClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_button_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkButton),
-	16,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_button_init,
-      };
-
-      button_type = g_type_register_static (GTK_TYPE_BIN, "GtkButton",
-					    &button_info, 0);
-    }
-
-  return button_type;
-}
+G_DEFINE_TYPE (GtkButton, gtk_button, GTK_TYPE_BIN)
 
 static void
 gtk_button_class_init (GtkButtonClass *klass)
@@ -185,8 +154,6 @@ gtk_button_class_init (GtkButtonClass *klass)
   widget_class = (GtkWidgetClass*) klass;
   container_class = (GtkContainerClass*) klass;
   
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->constructor = gtk_button_constructor;
   gobject_class->set_property = gtk_button_set_property;
   gobject_class->get_property = gtk_button_get_property;
@@ -323,7 +290,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    * @Deprecated: Use the GtkWidget::button-press-event signal.
    */ 
   button_signals[PRESSED] =
-    g_signal_new ("pressed",
+    g_signal_new (I_("pressed"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, pressed),
@@ -340,7 +307,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    * @Deprecated: Use the GtkWidget::button-release-event signal.
    */ 
   button_signals[RELEASED] =
-    g_signal_new ("released",
+    g_signal_new (I_("released"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, released),
@@ -355,7 +322,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    * Emitted when the button has been activated (pressed and released).
    */ 
   button_signals[CLICKED] =
-    g_signal_new ("clicked",
+    g_signal_new (I_("clicked"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkButtonClass, clicked),
@@ -372,7 +339,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    * @Deprecated: Use the GtkWidget::enter-notify-event signal.
    */ 
   button_signals[ENTER] =
-    g_signal_new ("enter",
+    g_signal_new (I_("enter"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, enter),
@@ -389,7 +356,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    * @Deprecated: Use the GtkWidget::leave-notify-event signal.
    */ 
   button_signals[LEAVE] =
-    g_signal_new ("leave",
+    g_signal_new (I_("leave"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkButtonClass, leave),
@@ -407,7 +374,7 @@ gtk_button_class_init (GtkButtonClass *klass)
    * "clicked" signal.
    */
   button_signals[ACTIVATE] =
-    g_signal_new ("activate",
+    g_signal_new (I_("activate"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkButtonClass, activate),
@@ -461,13 +428,27 @@ gtk_button_class_init (GtkButtonClass *klass)
 						       FALSE,
 						       GTK_PARAM_READABLE));
 
+  /**
+   * GtkButton:inner-border:
+   *
+   * Sets the border between the button edges and child.
+   *
+   * Since: 2.10
+   */
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_boxed ("inner-border",
+                                                               P_("Inner Border"),
+                                                               P_("Border between button edges and child."),
+                                                               GTK_TYPE_BORDER,
+                                                               GTK_PARAM_READABLE));
+
   gtk_settings_install_property (g_param_spec_boolean ("gtk-button-images",
 						       P_("Show button images"),
 						       P_("Whether stock icons should be shown in buttons"),
 						       TRUE,
 						       GTK_PARAM_READWRITE));
-  
-  g_type_class_add_private (gobject_class, sizeof (GtkButtonPrivate));  
+
+  g_type_class_add_private (gobject_class, sizeof (GtkButtonPrivate));
 }
 
 static void
@@ -507,7 +488,7 @@ gtk_button_destroy (GtkObject *object)
       button->label_text = NULL;
     }
   
-  (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  (* GTK_OBJECT_CLASS (gtk_button_parent_class)->destroy) (object);
 }
 
 static GObject*
@@ -518,9 +499,9 @@ gtk_button_constructor (GType                  type,
   GObject *object;
   GtkButton *button;
 
-  object = (* G_OBJECT_CLASS (parent_class)->constructor) (type,
-							   n_construct_properties,
-							   construct_params);
+  object = (* G_OBJECT_CLASS (gtk_button_parent_class)->constructor) (type,
+								      n_construct_properties,
+								      construct_params);
 
   button = GTK_BUTTON (object);
   button->constructed = TRUE;
@@ -570,7 +551,7 @@ gtk_button_add (GtkContainer *container,
 {
   maybe_set_alignment (GTK_BUTTON (container), widget);
 
-  GTK_CONTAINER_CLASS (parent_class)->add (container, widget);
+  GTK_CONTAINER_CLASS (gtk_button_parent_class)->add (container, widget);
 }
 
 static void
@@ -935,7 +916,7 @@ gtk_button_unrealize (GtkWidget *widget)
       button->event_window = NULL;
     }
   
-  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  GTK_WIDGET_CLASS (gtk_button_parent_class)->unrealize (widget);
 }
 
 static void
@@ -943,7 +924,7 @@ gtk_button_map (GtkWidget *widget)
 {
   GtkButton *button = GTK_BUTTON (widget);
   
-  GTK_WIDGET_CLASS (parent_class)->map (widget);
+  GTK_WIDGET_CLASS (gtk_button_parent_class)->map (widget);
 
   if (button->event_window)
     gdk_window_show (button->event_window);
@@ -957,13 +938,14 @@ gtk_button_unmap (GtkWidget *widget)
   if (button->event_window)
     gdk_window_hide (button->event_window);
 
-  GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+  GTK_WIDGET_CLASS (gtk_button_parent_class)->unmap (widget);
 }
 
 static void
 gtk_button_get_props (GtkButton *button,
 		      GtkBorder *default_border,
 		      GtkBorder *default_outside_border,
+                      GtkBorder *inner_border,
 		      gboolean  *interior_focus)
 {
   GtkWidget *widget =  GTK_WIDGET (button);
@@ -995,6 +977,19 @@ gtk_button_get_props (GtkButton *button,
 	*default_outside_border = default_default_outside_border;
     }
 
+  if (inner_border)
+    {
+      gtk_widget_style_get (widget, "inner-border", &tmp_border, NULL);
+
+      if (tmp_border)
+	{
+	  *inner_border = *tmp_border;
+	  g_free (tmp_border);
+	}
+      else
+	*inner_border = default_inner_border;
+    }
+
   if (interior_focus)
     gtk_widget_style_get (widget, "interior-focus", interior_focus, NULL);
 }
@@ -1005,19 +1000,22 @@ gtk_button_size_request (GtkWidget      *widget,
 {
   GtkButton *button = GTK_BUTTON (widget);
   GtkBorder default_border;
+  GtkBorder inner_border;
   gint focus_width;
   gint focus_pad;
 
-  gtk_button_get_props (button, &default_border, NULL, NULL);
+  gtk_button_get_props (button, &default_border, NULL, &inner_border, NULL);
   gtk_widget_style_get (GTK_WIDGET (widget),
 			"focus-line-width", &focus_width,
 			"focus-padding", &focus_pad,
 			NULL);
  
-  requisition->width = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
-			GTK_WIDGET (widget)->style->xthickness) * 2;
-  requisition->height = (GTK_CONTAINER (widget)->border_width + CHILD_SPACING +
-			 GTK_WIDGET (widget)->style->ythickness) * 2;
+  requisition->width = ((GTK_CONTAINER (widget)->border_width +
+                         GTK_WIDGET (widget)->style->xthickness) * 2 +
+                        inner_border.left + inner_border.right);
+  requisition->height = ((GTK_CONTAINER (widget)->border_width +
+                          GTK_WIDGET (widget)->style->ythickness) * 2 +
+                         inner_border.top + inner_border.bottom);
 
   if (GTK_WIDGET_CAN_DEFAULT (widget))
     {
@@ -1050,10 +1048,11 @@ gtk_button_size_allocate (GtkWidget     *widget,
   gint xthickness = GTK_WIDGET (widget)->style->xthickness;
   gint ythickness = GTK_WIDGET (widget)->style->ythickness;
   GtkBorder default_border;
+  GtkBorder inner_border;
   gint focus_width;
   gint focus_pad;
 
-  gtk_button_get_props (button, &default_border, NULL, NULL);
+  gtk_button_get_props (button, &default_border, NULL, &inner_border, NULL);
   gtk_widget_style_get (GTK_WIDGET (widget),
 			"focus-line-width", &focus_width,
 			"focus-padding", &focus_pad,
@@ -1071,12 +1070,18 @@ gtk_button_size_allocate (GtkWidget     *widget,
 
   if (GTK_BIN (button)->child && GTK_WIDGET_VISIBLE (GTK_BIN (button)->child))
     {
-      child_allocation.x = widget->allocation.x + border_width + CHILD_SPACING + xthickness;
-      child_allocation.y = widget->allocation.y + border_width + CHILD_SPACING + ythickness;
+      child_allocation.x = widget->allocation.x + border_width + inner_border.left + xthickness;
+      child_allocation.y = widget->allocation.y + border_width + inner_border.top + ythickness;
       
-      child_allocation.width = MAX (1, widget->allocation.width - (CHILD_SPACING + xthickness) * 2 -
+      child_allocation.width = MAX (1, widget->allocation.width -
+                                    xthickness * 2 -
+                                    inner_border.left -
+                                    inner_border.right -
 				    border_width * 2);
-      child_allocation.height = MAX (1, widget->allocation.height - (CHILD_SPACING + ythickness) * 2 -
+      child_allocation.height = MAX (1, widget->allocation.height -
+                                     ythickness * 2 -
+                                     inner_border.top -
+                                     inner_border.bottom -
 				     border_width * 2);
 
       if (GTK_WIDGET_CAN_DEFAULT (button))
@@ -1135,7 +1140,7 @@ _gtk_button_paint (GtkButton    *button,
       widget = GTK_WIDGET (button);
       border_width = GTK_CONTAINER (widget)->border_width;
 
-      gtk_button_get_props (button, &default_border, &default_outside_border, &interior_focus);
+      gtk_button_get_props (button, &default_border, &default_outside_border, NULL, &interior_focus);
       gtk_widget_style_get (GTK_WIDGET (widget),
 			    "focus-line-width", &focus_width,
 			    "focus-padding", &focus_pad,
@@ -1235,7 +1240,7 @@ gtk_button_expose (GtkWidget      *widget,
 			 button->depressed ? GTK_SHADOW_IN : GTK_SHADOW_OUT,
 			 "button", "buttondefault");
       
-      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
+      (* GTK_WIDGET_CLASS (gtk_button_parent_class)->expose_event) (widget, event);
     }
   
   return FALSE;
@@ -1310,8 +1315,8 @@ gtk_button_key_release (GtkWidget   *widget,
       gtk_button_finish_activate (button, TRUE);
       return TRUE;
     }
-  else if (GTK_WIDGET_CLASS (parent_class)->key_release_event)
-    return GTK_WIDGET_CLASS (parent_class)->key_release_event (widget, event);
+  else if (GTK_WIDGET_CLASS (gtk_button_parent_class)->key_release_event)
+    return GTK_WIDGET_CLASS (gtk_button_parent_class)->key_release_event (widget, event);
   else
     return FALSE;
 }
@@ -1808,7 +1813,7 @@ gtk_button_screen_changed (GtkWidget *widget,
     g_signal_connect (settings, "notify::gtk-button-images",
 		      G_CALLBACK (gtk_button_setting_changed), 0);
   g_object_set_data (G_OBJECT (settings), 
-		     "gtk-button-connection",
+		     I_("gtk-button-connection"),
 		     GUINT_TO_POINTER (show_image_connection));
 
   show_image_change_notify (GTK_BUTTON (widget));
@@ -1862,7 +1867,7 @@ gtk_button_set_image (GtkButton *button,
   g_return_if_fail (image == NULL || GTK_IS_WIDGET (image));
 
   priv = GTK_BUTTON_GET_PRIVATE (button);
- 
+
   priv->image = image;
   priv->image_is_stock = (image == NULL);
 
