@@ -119,7 +119,7 @@ enum
 
 G_DEFINE_TYPE_WITH_CODE (GtkCellView, gtk_cell_view, GTK_TYPE_WIDGET, 
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_CELL_LAYOUT,
-						gtk_cell_view_cell_layout_init));
+						gtk_cell_view_cell_layout_init))
 
 static void
 gtk_cell_view_class_init (GtkCellViewClass *klass)
