@@ -64,7 +64,7 @@ struct _GtkEntryCompletionClass
   void     (* action_activated) (GtkEntryCompletion *completion,
                                  gint                index_);
   gboolean (* insert_prefix)    (GtkEntryCompletion *completion,
-				 gchar              *prefix); 
+				 const gchar        *prefix); 
 
   /* Padding for future expansion */
   void (*_gtk_reserved0) (void);
@@ -73,7 +73,7 @@ struct _GtkEntryCompletionClass
 };
 
 /* core */
-GType               gtk_entry_completion_get_type               (void);
+GType               gtk_entry_completion_get_type               (void) G_GNUC_CONST;
 GtkEntryCompletion *gtk_entry_completion_new                    (void);
 
 GtkWidget          *gtk_entry_completion_get_entry              (GtkEntryCompletion          *completion);
@@ -102,11 +102,18 @@ void                gtk_entry_completion_delete_action          (GtkEntryComplet
                                                                  gint                         index_);
 
 void                gtk_entry_completion_set_inline_completion  (GtkEntryCompletion          *completion,
-                                                                 gboolean inline_completion);
+                                                                 gboolean                     inline_completion);
 gboolean            gtk_entry_completion_get_inline_completion  (GtkEntryCompletion          *completion);
 void                gtk_entry_completion_set_popup_completion   (GtkEntryCompletion          *completion,
-                                                                 gboolean popup_completion);
+                                                                 gboolean                     popup_completion);
 gboolean            gtk_entry_completion_get_popup_completion   (GtkEntryCompletion          *completion);
+void                gtk_entry_completion_set_popup_set_width    (GtkEntryCompletion          *completion,
+                                                                 gboolean                     popup_set_width);
+gboolean            gtk_entry_completion_get_popup_set_width    (GtkEntryCompletion          *completion);
+void                gtk_entry_completion_set_popup_single_match (GtkEntryCompletion          *completion,
+                                                                 gboolean                     popup_single_match);
+gboolean            gtk_entry_completion_get_popup_single_match (GtkEntryCompletion          *completion);
+
 
 /* convenience */
 void                gtk_entry_completion_set_text_column        (GtkEntryCompletion          *completion,
