@@ -39,18 +39,12 @@ gtk_file_chooser_get_type (void)
 
   if (!file_chooser_type)
     {
-      static const GTypeInfo file_chooser_info =
-      {
-	sizeof (GtkFileChooserIface),  /* class_size */
-	NULL,                          /* base_init */
-	NULL,			       /* base_finalize */
-	(GClassInitFunc)gtk_file_chooser_class_init, /* class_init */
-      };
-
-      file_chooser_type = g_type_register_static (G_TYPE_INTERFACE,
-						  "GtkFileChooser",
-						  &file_chooser_info, 0);
-
+      file_chooser_type = g_type_register_static_simple (G_TYPE_INTERFACE,
+							 I_("GtkFileChooser"),
+							 sizeof (GtkFileChooserIface),
+							 gtk_file_chooser_class_init,
+							 0, NULL, 0);
+      
       g_type_interface_add_prerequisite (file_chooser_type, GTK_TYPE_WIDGET);
     }
 
@@ -96,7 +90,7 @@ gtk_file_chooser_class_init (gpointer g_iface)
    * gtk_file_chooser_set_current_folder_uri(),
    * gtk_file_chooser_get_current_folder_uri().
    */
-  g_signal_new ("current-folder-changed",
+  g_signal_new (I_("current-folder-changed"),
 		iface_type,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GtkFileChooserIface, current_folder_changed),
@@ -123,7 +117,7 @@ gtk_file_chooser_class_init (gpointer g_iface)
    * gtk_file_chooser_unselect_uri(), gtk_file_chooser_get_uri(),
    * gtk_file_chooser_get_uris().
    */
-  g_signal_new ("selection-changed",
+  g_signal_new (I_("selection-changed"),
 		iface_type,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GtkFileChooserIface, selection_changed),
@@ -157,7 +151,7 @@ gtk_file_chooser_class_init (gpointer g_iface)
    * gtk_file_chooser_get_preview_filename(),
    * gtk_file_chooser_get_preview_uri().
    */
-  g_signal_new ("update-preview",
+  g_signal_new (I_("update-preview"),
 		iface_type,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GtkFileChooserIface, update_preview),
@@ -181,7 +175,7 @@ gtk_file_chooser_class_init (gpointer g_iface)
    * gtk_file_chooser_get_filenames(), gtk_file_chooser_get_uri(),
    * gtk_file_chooser_get_uris().
    */
-  g_signal_new ("file-activated",
+  g_signal_new (I_("file-activated"),
 		iface_type,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GtkFileChooserIface, file_activated),
@@ -190,7 +184,7 @@ gtk_file_chooser_class_init (gpointer g_iface)
 		G_TYPE_NONE, 0);
 
   /* Documented in the docbook files */
-  g_signal_new ("confirm-overwrite",
+  g_signal_new (I_("confirm-overwrite"),
 		iface_type,
 		G_SIGNAL_RUN_LAST,
 		G_STRUCT_OFFSET (GtkFileChooserIface, confirm_overwrite),
@@ -260,22 +254,22 @@ gtk_file_chooser_class_init (gpointer g_iface)
 							     P_("Whether the hidden files and folders should be displayed"),
 							     FALSE,
 							     GTK_PARAM_READWRITE));
-  
+
   /**
    * GtkFileChooser:do-overwrite-confirmation:
-   *
-   * Whether a file chooser in GTK_FILE_CHOOSER_ACTION_SAVE
+   * 
+   * Whether a file chooser in %GTK_FILE_CHOOSER_ACTION_SAVE mode
    * will present an overwrite confirmation dialog if the user
    * selects a file name that already exists.
-   * 
+   *
    * Since: 2.8
    */
   g_object_interface_install_property (g_iface,
 				       g_param_spec_boolean ("do-overwrite-confirmation",
 							     P_("Do overwrite confirmation"),
-							     P_("Whether a file chooser in GTK_FILE_CHOOSER_ACTION_SAVE "
-								"will present an overwrite confirmation dialog if the user "
-								"selects a file name that already exists."),
+							     P_("Whether a file chooser in save mode "
+								"will present an overwrite confirmation dialog "
+								"if necessary."),
 							     FALSE,
 							     GTK_PARAM_READWRITE));
 }
@@ -292,10 +286,7 @@ gtk_file_chooser_class_init (gpointer g_iface)
 GQuark
 gtk_file_chooser_error_quark (void)
 {
-  static GQuark quark = 0;
-  if (quark == 0)
-    quark = g_quark_from_static_string ("gtk-file-chooser-error-quark");
-  return quark;
+  return g_quark_from_static_string ("gtk-file-chooser-error-quark");
 }
 
 /**
@@ -720,6 +711,9 @@ gtk_file_chooser_get_current_folder (GtkFileChooser *chooser)
   file_system = _gtk_file_chooser_get_file_system (chooser);
 
   path = _gtk_file_chooser_get_current_folder_path (chooser);
+  if (!path)
+    return NULL;
+
   filename = gtk_file_system_path_to_filename (file_system, path);
   gtk_file_path_free (path);
 
