@@ -51,9 +51,6 @@ struct _GtkImagePrivate
 
 
 #define DEFAULT_ICON_SIZE GTK_ICON_SIZE_BUTTON
-
-static void gtk_image_class_init   (GtkImageClass  *klass);
-static void gtk_image_init         (GtkImage       *image);
 static gint gtk_image_expose       (GtkWidget      *widget,
                                     GdkEventExpose *event);
 static void gtk_image_unmap        (GtkWidget      *widget);
@@ -83,8 +80,6 @@ static void gtk_image_get_property      (GObject          *object,
 
 static void icon_theme_changed          (GtkImage         *image);
 
-static gpointer parent_class;
-
 enum
 {
   PROP_0,
@@ -102,32 +97,7 @@ enum
   PROP_STORAGE_TYPE
 };
 
-GType
-gtk_image_get_type (void)
-{
-  static GType image_type = 0;
-
-  if (!image_type)
-    {
-      static const GTypeInfo image_info =
-      {
-	sizeof (GtkImageClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_image_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkImage),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_image_init,
-      };
-
-      image_type = g_type_register_static (GTK_TYPE_MISC, "GtkImage",
-					   &image_info, 0);
-    }
-
-  return image_type;
-}
+G_DEFINE_TYPE (GtkImage, gtk_image, GTK_TYPE_MISC)
 
 static void
 gtk_image_class_init (GtkImageClass *class)
@@ -136,8 +106,6 @@ gtk_image_class_init (GtkImageClass *class)
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class = G_OBJECT_CLASS (class);
   
   gobject_class->set_property = gtk_image_set_property;
@@ -298,7 +266,7 @@ gtk_image_destroy (GtkObject *object)
 
   gtk_image_reset (image);
   
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+  GTK_OBJECT_CLASS (gtk_image_parent_class)->destroy (object);
 }
 
 static void 
@@ -308,10 +276,8 @@ gtk_image_set_property (GObject      *object,
 			GParamSpec   *pspec)
 {
   GtkImage *image;
-  GtkImagePrivate *priv;
 
   image = GTK_IMAGE (object);
-  priv = GTK_IMAGE_GET_PRIVATE (image);
   
   switch (prop_id)
     {
@@ -611,7 +577,7 @@ gtk_image_new_from_pixbuf (GdkPixbuf *pixbuf)
  * @size: a stock icon size
  * 
  * Creates a #GtkImage displaying a stock icon. Sample stock icon
- * names are #GTK_STOCK_OPEN, #GTK_STOCK_EXIT. Sample stock sizes
+ * names are #GTK_STOCK_OPEN, #GTK_STOCK_QUIT. Sample stock sizes
  * are #GTK_ICON_SIZE_MENU, #GTK_ICON_SIZE_SMALL_TOOLBAR. If the stock
  * icon name isn't known, the image will be empty.
  * You can register your own stock icon names, see
@@ -673,7 +639,12 @@ gtk_image_new_from_icon_set (GtkIconSet     *icon_set,
  * The #GtkImage does not assume a reference to the
  * animation; you still need to unref it if you own references.
  * #GtkImage will add its own reference rather than adopting yours.
- * 
+ *
+ * Note that the animation frames are shown using a timeout with
+ * #G_PRIORITY_DEFAULT. When using animations to indicate busyness,
+ * keep in mind that the animation will only be shown if the main loop
+ * is not busy with something that has a higher priority.
+ *
  * Return value: a new #GtkImage widget
  **/
 GtkWidget*
@@ -1367,8 +1338,8 @@ gtk_image_unmap (GtkWidget *widget)
 {
   gtk_image_reset_anim_iter (GTK_IMAGE (widget));
 
-  if (GTK_WIDGET_CLASS (parent_class)->unmap)
-    GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->unmap)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->unmap (widget);
 }
 
 static void
@@ -1376,8 +1347,8 @@ gtk_image_unrealize (GtkWidget *widget)
 {
   gtk_image_reset_anim_iter (GTK_IMAGE (widget));
 
-  if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-    GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->unrealize)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->unrealize (widget);
 }
 
 static gint
@@ -1398,9 +1369,12 @@ animation_timeout (gpointer data)
       g_timeout_add (gdk_pixbuf_animation_iter_get_delay_time (image->data.anim.iter),
                      animation_timeout,
                      image);
-  
+
   gtk_widget_queue_draw (GTK_WIDGET (image));
 
+  if (GTK_WIDGET_DRAWABLE (image))
+    gdk_window_process_updates (GTK_WIDGET (image)->window, TRUE);
+
   GDK_THREADS_LEAVE ();
 
   return FALSE;
@@ -1409,14 +1383,11 @@ animation_timeout (gpointer data)
 static void
 icon_theme_changed (GtkImage *image)
 {
-  if (image->storage_type == GTK_IMAGE_ICON_NAME) 
-    {
-      if (image->data.name.pixbuf)
-	g_object_unref (image->data.name.pixbuf);
-      image->data.name.pixbuf = NULL;
+  if (image->data.name.pixbuf)
+    g_object_unref (image->data.name.pixbuf);
+  image->data.name.pixbuf = NULL;
 
-      gtk_widget_queue_draw (GTK_WIDGET (image));
-    }
+  gtk_widget_queue_draw (GTK_WIDGET (image));
 }
 
 static void
@@ -1777,17 +1748,19 @@ gtk_image_expose (GtkWidget      *widget,
                 }
 
               if (pixbuf)
-                gdk_draw_pixbuf (widget->window,
-			         widget->style->black_gc,
-				 pixbuf,
-				 image_bound.x - x,
-				 image_bound.y - y,
-				 image_bound.x,
-				 image_bound.y,
-				 image_bound.width,
-				 image_bound.height,
-				 GDK_RGB_DITHER_NORMAL,
-				 0, 0);
+                {
+                  gdk_draw_pixbuf (widget->window,
+				   widget->style->black_gc,
+				   pixbuf,
+				   image_bound.x - x,
+				   image_bound.y - y,
+				   image_bound.x,
+				   image_bound.y,
+				   image_bound.width,
+				   image_bound.height,
+				   GDK_RGB_DITHER_NORMAL,
+				   0, 0);
+                }
             }
           else
             {
@@ -1828,10 +1801,10 @@ gtk_image_expose (GtkWidget      *widget,
           gdk_gc_set_clip_mask (widget->style->black_gc, NULL);
           gdk_gc_set_clip_origin (widget->style->black_gc, 0, 0);
         }
-
-      if (pixbuf)
-        g_object_unref (pixbuf);
       
+      if (pixbuf)
+	g_object_unref (pixbuf);
+
     } /* if widget is drawable */
 
   return FALSE;
@@ -1842,8 +1815,6 @@ gtk_image_reset (GtkImage *image)
 {
   GtkImagePrivate *priv;
 
-  g_return_if_fail (GTK_IS_IMAGE (image));
-
   priv = GTK_IMAGE_GET_PRIVATE (image);
 
   g_object_freeze_notify (G_OBJECT (image));
@@ -2029,7 +2000,7 @@ gtk_image_size_request (GtkWidget      *widget,
   gtk_image_calc_size (image);
 
   /* Chain up to default that simply reads current requisition */
-  GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+  GTK_WIDGET_CLASS (gtk_image_parent_class)->size_request (widget, requisition);
 }
 
 static void
@@ -2040,8 +2011,8 @@ gtk_image_style_set (GtkWidget      *widget,
 
   image = GTK_IMAGE (widget);
 
-  if (GTK_WIDGET_CLASS (parent_class)->style_set)
-    GTK_WIDGET_CLASS (parent_class)->style_set (widget, prev_style);
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->style_set)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->style_set (widget, prev_style);
   
   icon_theme_changed (image);
 }
@@ -2054,8 +2025,8 @@ gtk_image_screen_changed (GtkWidget *widget,
 
   image = GTK_IMAGE (widget);
 
-  if (GTK_WIDGET_CLASS (parent_class)->screen_changed)
-    GTK_WIDGET_CLASS (parent_class)->screen_changed (widget, prev_screen);
+  if (GTK_WIDGET_CLASS (gtk_image_parent_class)->screen_changed)
+    GTK_WIDGET_CLASS (gtk_image_parent_class)->screen_changed (widget, prev_screen);
 
   icon_theme_changed (image);
 }
