@@ -144,8 +144,6 @@ static GList *init_functions = NULL;	   /* A list of init functions.
 					    */
 static GList *quit_functions = NULL;	   /* A list of quit functions.
 					    */
-static GMemChunk *quit_mem_chunk = NULL;
-
 static GSList *key_snoopers = NULL;
 
 guint gtk_debug_flags = 0;		   /* Global GTK debug flag */
@@ -163,9 +161,6 @@ static const GDebugKey gtk_debug_keys[] = {
   {"geometry", GTK_DEBUG_GEOMETRY},
   {"icontheme", GTK_DEBUG_ICONTHEME}
 };
-
-static const guint gtk_ndebug_keys = sizeof (gtk_debug_keys) / sizeof (GDebugKey);
-
 #endif /* G_ENABLE_DEBUG */
 
 /**
@@ -372,7 +367,7 @@ gtk_arg_debug_cb (const char *key, const char *value, gpointer user_data)
 {
   gtk_debug_flags |= g_parse_debug_string (value,
 					   gtk_debug_keys,
-					   gtk_ndebug_keys);
+					   G_N_ELEMENTS (gtk_debug_keys));
 
   return TRUE;
 }
@@ -382,7 +377,7 @@ gtk_arg_no_debug_cb (const char *key, const char *value, gpointer user_data)
 {
   gtk_debug_flags &= ~g_parse_debug_string (value,
 					    gtk_debug_keys,
-					    gtk_ndebug_keys);
+					    G_N_ELEMENTS (gtk_debug_keys));
 
   return TRUE;
 }
@@ -404,7 +399,7 @@ gtk_arg_module_cb (const char *key, const char *value, gpointer user_data)
   return TRUE;
 }
 
-static GOptionEntry gtk_args[] = {
+static const GOptionEntry gtk_args[] = {
   { "gtk-module",       0, 0, G_OPTION_ARG_CALLBACK, gtk_arg_module_cb,   
     /* Description of --gtk-module=MODULES in --help output */ N_("Load additional GTK+ modules"), 
     /* Placeholder in --gtk-module=MODULES in --help output */ N_("MODULES") },
@@ -449,7 +444,7 @@ do_pre_parse_initialization (int    *argc,
     {
       gtk_debug_flags = g_parse_debug_string (env_string,
 					      gtk_debug_keys,
-					      gtk_ndebug_keys);
+					      G_N_ELEMENTS (gtk_debug_keys));
       env_string = NULL;
     }
 #endif	/* G_ENABLE_DEBUG */
@@ -548,6 +543,7 @@ post_parse_hook (GOptionContext *context,
 {
   OptionGroupInfo *info = data;
 
+  
   do_post_parse_initialization (NULL, NULL);
   
   if (info->open_default_display)
@@ -1283,7 +1279,7 @@ gtk_main_do_event (GdkEvent *event)
    *  in the user_data field of GdkWindow's.
    *  Ignore the event if we don't have a widget for it, except
    *  for GDK_PROPERTY_NOTIFY events which are handled specialy.
-   *  Though this happens rarely, bogus events can occur
+   *  Though this happens rarely, bogus events can occour
    *  for e.g. destroyed GdkWindows. 
    */
   event_widget = gtk_get_event_widget (event);
@@ -1491,9 +1487,9 @@ gtk_main_get_window_group (GtkWidget   *widget)
     toplevel = gtk_widget_get_toplevel (widget);
 
   if (toplevel && GTK_IS_WINDOW (toplevel))
-    return _gtk_window_get_group (GTK_WINDOW (toplevel));
+    return gtk_window_get_group (GTK_WINDOW (toplevel));
   else
-    return _gtk_window_get_group (NULL);
+    return gtk_window_get_group (NULL);
 }
 
 typedef struct
@@ -1528,7 +1524,7 @@ gtk_grab_notify_foreach (GtkWidget *child,
     _gtk_widget_grab_notify (child, was_shadowed);
   
   if ((was_shadowed || is_shadowed) && GTK_IS_CONTAINER (child))
-    gtk_container_forall (GTK_CONTAINER (child), gtk_grab_notify_foreach, info);
+    gtk_container_foreach (GTK_CONTAINER (child), gtk_grab_notify_foreach, info);
       
   g_object_unref (child);
   
@@ -1563,7 +1559,7 @@ gtk_grab_notify (GtkWindowGroup *group,
       info.was_grabbed = FALSE;
       info.is_grabbed = FALSE;
 
-      if (group == _gtk_window_get_group (toplevel))
+      if (group == gtk_window_get_group (toplevel))
 	gtk_container_foreach (GTK_CONTAINER (toplevel), 
 			       gtk_grab_notify_foreach, &info);
       g_object_unref (toplevel);
@@ -1587,10 +1583,10 @@ gtk_grab_add (GtkWidget *widget)
       group = gtk_main_get_window_group (widget);
 
       if (group->grabs)
- 	old_grab_widget = (GtkWidget *)group->grabs->data;
+	old_grab_widget = (GtkWidget *)group->grabs->data;
       else
 	old_grab_widget = NULL;
-      
+
       g_object_ref (widget);
       group->grabs = g_slist_prepend (group->grabs, widget);
 
@@ -1722,11 +1718,7 @@ gtk_quit_add_full (guint		main_level,
   
   g_return_val_if_fail ((function != NULL) || (marshal != NULL), 0);
 
-  if (!quit_mem_chunk)
-    quit_mem_chunk = g_mem_chunk_new ("quit mem chunk", sizeof (GtkQuitFunction),
-				      512, G_ALLOC_AND_FREE);
-  
-  quitf = g_chunk_new (GtkQuitFunction, quit_mem_chunk);
+  quitf = g_slice_new (GtkQuitFunction);
   
   quitf->id = quit_id++;
   quitf->main_level = main_level;
@@ -1745,7 +1737,7 @@ gtk_quit_destroy (GtkQuitFunction *quitf)
 {
   if (quitf->destroy)
     quitf->destroy (quitf->data);
-  g_mem_chunk_free (quit_mem_chunk, quitf);
+  g_slice_free (GtkQuitFunction, quitf);
 }
 
 static gint
