@@ -28,6 +28,7 @@
 #include <config.h>
 #include <stdio.h>
 #include <math.h>
+#include <gdk/gdkkeysyms.h>
 #include "gtkintl.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
@@ -37,15 +38,16 @@
 #include "gtkprivate.h"
 #include "gtkalias.h"
 
-#define SCROLL_INITIAL_DELAY 250  /* must hold button this long before ... */
-#define SCROLL_LATER_DELAY   100  /* ... it starts repeating at this rate  */
-#define UPDATE_DELAY         300  /* Delay for queued update */
+#define SCROLL_DELAY_FACTOR 5    /* Scroll repeat multiplier */
+#define UPDATE_DELAY        300  /* Delay for queued update */
 
 enum {
   PROP_0,
   PROP_UPDATE_POLICY,
   PROP_ADJUSTMENT,
-  PROP_INVERTED
+  PROP_INVERTED,
+  PROP_LOWER_STEPPER_SENSITIVITY,
+  PROP_UPPER_STEPPER_SENSITIVITY
 };
 
 enum {
@@ -89,11 +91,13 @@ struct _GtkRangeLayout
   /* "grabbed" mouse location, OUTSIDE for no grab */
   MouseLocation grab_location;
   gint grab_button; /* 0 if none */
+
+  /* Stepper sensitivity */
+  GtkSensitivityType lower_sensitivity;
+  GtkSensitivityType upper_sensitivity;
 };
 
 
-static void gtk_range_class_init     (GtkRangeClass    *klass);
-static void gtk_range_init           (GtkRange         *range);
 static void gtk_range_set_property   (GObject          *object,
                                       guint             prop_id,
                                       const GValue     *value,
@@ -181,39 +185,13 @@ static gboolean      gtk_range_real_change_value        (GtkRange      *range,
                                                          GtkScrollType  scroll,
                                                          gdouble        value);
 static void          gtk_range_update_value             (GtkRange      *range);
+static gboolean      gtk_range_key_press                (GtkWidget     *range,
+							 GdkEventKey   *event);
 
 
-static GtkWidgetClass *parent_class = NULL;
 static guint signals[LAST_SIGNAL];
 
-
-GType
-gtk_range_get_type (void)
-{
-  static GType range_type = 0;
-
-  if (!range_type)
-    {
-      static const GTypeInfo range_info =
-      {
-	sizeof (GtkRangeClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_range_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkRange),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_range_init,
-	NULL,		/* value_table */
-      };
-
-      range_type = g_type_register_static (GTK_TYPE_WIDGET, "GtkRange",
-					   &range_info, G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return range_type;
-}
+G_DEFINE_ABSTRACT_TYPE (GtkRange, gtk_range, GTK_TYPE_WIDGET)
 
 static void
 gtk_range_class_init (GtkRangeClass *class)
@@ -226,8 +204,6 @@ gtk_range_class_init (GtkRangeClass *class)
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
 
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class->set_property = gtk_range_set_property;
   gobject_class->get_property = gtk_range_get_property;
   gobject_class->finalize = gtk_range_finalize;
@@ -250,6 +226,7 @@ gtk_range_class_init (GtkRangeClass *class)
   widget_class->grab_notify = gtk_range_grab_notify;
   widget_class->state_changed = gtk_range_state_changed;
   widget_class->style_set = gtk_range_style_set;
+  widget_class->key_press_event = gtk_range_key_press;
 
   class->move_slider = gtk_range_move_slider;
   class->change_value = gtk_range_real_change_value;
@@ -258,7 +235,7 @@ gtk_range_class_init (GtkRangeClass *class)
   class->stepper_detail = "stepper";
 
   signals[VALUE_CHANGED] =
-    g_signal_new ("value_changed",
+    g_signal_new (I_("value_changed"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkRangeClass, value_changed),
@@ -267,7 +244,7 @@ gtk_range_class_init (GtkRangeClass *class)
                   G_TYPE_NONE, 0);
   
   signals[ADJUST_BOUNDS] =
-    g_signal_new ("adjust_bounds",
+    g_signal_new (I_("adjust_bounds"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkRangeClass, adjust_bounds),
@@ -277,7 +254,7 @@ gtk_range_class_init (GtkRangeClass *class)
                   G_TYPE_DOUBLE);
   
   signals[MOVE_SLIDER] =
-    g_signal_new ("move_slider",
+    g_signal_new (I_("move_slider"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                   G_STRUCT_OFFSET (GtkRangeClass, move_slider),
@@ -313,7 +290,7 @@ gtk_range_class_init (GtkRangeClass *class)
    * Since: 2.6
    */
   signals[CHANGE_VALUE] =
-    g_signal_new ("change_value",
+    g_signal_new (I_("change_value"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GtkRangeClass, change_value),
@@ -347,7 +324,25 @@ gtk_range_class_init (GtkRangeClass *class)
 							P_("Invert direction slider moves to increase range value"),
                                                          FALSE,
                                                          GTK_PARAM_READWRITE));
-  
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_LOWER_STEPPER_SENSITIVITY,
+                                   g_param_spec_enum ("lower-stepper-sensitivity",
+						      P_("Lower stepper sensitivity"),
+						      P_("The sensitivity policy for the stepper that points to the adjustment's lower side"),
+						      GTK_TYPE_SENSITIVITY_TYPE,
+						      GTK_SENSITIVITY_AUTO,
+						      GTK_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_UPPER_STEPPER_SENSITIVITY,
+                                   g_param_spec_enum ("upper-stepper-sensitivity",
+						      P_("Upper stepper sensitivity"),
+						      P_("The sensitivity policy for the stepper that points to the adjustment's upper side"),
+						      GTK_TYPE_SENSITIVITY_TYPE,
+						      GTK_SENSITIVITY_AUTO,
+						      GTK_PARAM_READWRITE));
+
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("slider-width",
 							     P_("Slider Width"),
@@ -396,6 +391,13 @@ gtk_range_class_init (GtkRangeClass *class)
 							     G_MAXINT,
 							     0,
 							     GTK_PARAM_READABLE));
+
+  gtk_widget_class_install_style_property (widget_class,
+					   g_param_spec_boolean ("activate_slider",
+                                                                 P_("Draw slider ACTIVE during drag"),
+							         P_("With this option set to TRUE, sliders will be drawn ACTIVE and with shadow IN while they are dragged"),
+							         FALSE,
+							         G_PARAM_READABLE));
 }
 
 static void
@@ -419,6 +421,12 @@ gtk_range_set_property (GObject      *object,
     case PROP_INVERTED:
       gtk_range_set_inverted (range, g_value_get_boolean (value));
       break;
+    case PROP_LOWER_STEPPER_SENSITIVITY:
+      gtk_range_set_lower_stepper_sensitivity (range, g_value_get_enum (value));
+      break;
+    case PROP_UPPER_STEPPER_SENSITIVITY:
+      gtk_range_set_upper_stepper_sensitivity (range, g_value_get_enum (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -446,6 +454,12 @@ gtk_range_get_property (GObject      *object,
     case PROP_INVERTED:
       g_value_set_boolean (value, range->inverted);
       break;
+    case PROP_LOWER_STEPPER_SENSITIVITY:
+      g_value_set_enum (value, gtk_range_get_lower_stepper_sensitivity (range));
+      break;
+    case PROP_UPPER_STEPPER_SENSITIVITY:
+      g_value_set_enum (value, gtk_range_get_upper_stepper_sensitivity (range));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -474,6 +488,8 @@ gtk_range_init (GtkRange *range)
   range->layout->mouse_y = -1;
   range->layout->grab_location = MOUSE_OUTSIDE;
   range->layout->grab_button = 0;
+  range->layout->lower_sensitivity = GTK_SENSITIVITY_AUTO;
+  range->layout->upper_sensitivity = GTK_SENSITIVITY_AUTO;
   range->timer = NULL;  
 }
 
@@ -582,8 +598,7 @@ gtk_range_set_adjustment (GtkRange      *range,
 	}
 
       range->adjustment = adjustment;
-      g_object_ref (adjustment);
-      gtk_object_sink (GTK_OBJECT (adjustment));
+      g_object_ref_sink (adjustment);
       
       g_signal_connect (adjustment, "changed",
 			G_CALLBACK (gtk_range_adjustment_changed),
@@ -640,6 +655,92 @@ gtk_range_get_inverted (GtkRange *range)
   return range->inverted;
 }
 
+/**
+ * gtk_range_set_lower_stepper_sensitivity:
+ * @range:       a #GtkRange
+ * @sensitivity: the lower stepper's sensitivity policy.
+ *
+ * Sets the sensitivity policy for the stepper that points to the
+ * 'lower' end of the GtkRange's adjustment.
+ *
+ * Since: 2.10
+ **/
+void
+gtk_range_set_lower_stepper_sensitivity (GtkRange           *range,
+                                         GtkSensitivityType  sensitivity)
+{
+  g_return_if_fail (GTK_IS_RANGE (range));
+
+  if (range->layout->lower_sensitivity != sensitivity)
+    {
+      range->layout->lower_sensitivity = sensitivity;
+      gtk_widget_queue_draw (GTK_WIDGET (range));
+      g_object_notify (G_OBJECT (range), "lower-stepper-sensitivity");
+    }
+}
+
+/**
+ * gtk_range_get_lower_stepper_sensitivity:
+ * @range: a #GtkRange
+ *
+ * Gets the sensitivity policy for the stepper that points to the
+ * 'lower' end of the GtkRange's adjustment.
+ *
+ * Return value: The lower stepper's sensitivity policy.
+ *
+ * Since: 2.10
+ **/
+GtkSensitivityType
+gtk_range_get_lower_stepper_sensitivity (GtkRange *range)
+{
+  g_return_val_if_fail (GTK_IS_RANGE (range), GTK_SENSITIVITY_AUTO);
+
+  return range->layout->lower_sensitivity;
+}
+
+/**
+ * gtk_range_set_upper_stepper_sensitivity:
+ * @range:       a #GtkRange
+ * @sensitivity: the upper stepper's sensitivity policy.
+ *
+ * Sets the sensitivity policy for the stepper that points to the
+ * 'upper' end of the GtkRange's adjustment.
+ *
+ * Since: 2.10
+ **/
+void
+gtk_range_set_upper_stepper_sensitivity (GtkRange           *range,
+                                         GtkSensitivityType  sensitivity)
+{
+  g_return_if_fail (GTK_IS_RANGE (range));
+
+  if (range->layout->upper_sensitivity != sensitivity)
+    {
+      range->layout->upper_sensitivity = sensitivity;
+      gtk_widget_queue_draw (GTK_WIDGET (range));
+      g_object_notify (G_OBJECT (range), "upper-stepper-sensitivity");
+    }
+}
+
+/**
+ * gtk_range_get_upper_stepper_sensitivity:
+ * @range: a #GtkRange
+ *
+ * Gets the sensitivity policy for the stepper that points to the
+ * 'upper' end of the GtkRange's adjustment.
+ *
+ * Return value: The upper stepper's sensitivity policy.
+ *
+ * Since: 2.10
+ **/
+GtkSensitivityType
+gtk_range_get_upper_stepper_sensitivity (GtkRange *range)
+{
+  g_return_val_if_fail (GTK_IS_RANGE (range), GTK_SENSITIVITY_AUTO);
+
+  return range->layout->upper_sensitivity;
+}
+
 /**
  * gtk_range_set_increments:
  * @range: a #GtkRange
@@ -754,7 +855,7 @@ gtk_range_finalize (GObject *object)
 
   g_free (range->layout);
 
-  (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+  (* G_OBJECT_CLASS (gtk_range_parent_class)->finalize) (object);
 }
 
 static void
@@ -777,7 +878,7 @@ gtk_range_destroy (GtkObject *object)
       range->adjustment = NULL;
     }
 
-  (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  (* GTK_OBJECT_CLASS (gtk_range_parent_class)->destroy) (object);
 }
 
 static void
@@ -875,8 +976,8 @@ gtk_range_unrealize (GtkWidget *widget)
   gdk_window_destroy (range->event_window);
   range->event_window = NULL;
   
-  if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-    (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
+  if (GTK_WIDGET_CLASS (gtk_range_parent_class)->unrealize)
+    (* GTK_WIDGET_CLASS (gtk_range_parent_class)->unrealize) (widget);
 }
 
 static void
@@ -886,7 +987,7 @@ gtk_range_map (GtkWidget *widget)
   
   gdk_window_show (range->event_window);
 
-  GTK_WIDGET_CLASS (parent_class)->map (widget);
+  GTK_WIDGET_CLASS (gtk_range_parent_class)->map (widget);
 }
 
 static void
@@ -898,7 +999,7 @@ gtk_range_unmap (GtkWidget *widget)
 
   gdk_window_hide (range->event_window);
 
-  GTK_WIDGET_CLASS (parent_class)->unmap (widget);
+  GTK_WIDGET_CLASS (gtk_range_parent_class)->unmap (widget);
 }
 
 static void
@@ -919,14 +1020,57 @@ draw_stepper (GtkRange     *range,
   gint arrow_width;
   gint arrow_height;
 
+  gboolean arrow_sensitive = TRUE;
+
   /* More to get the right clip region than for efficiency */
   if (!gdk_rectangle_intersect (area, rect, &intersection))
     return;
 
   intersection.x += widget->allocation.x;
   intersection.y += widget->allocation.y;
-  
-  if (!GTK_WIDGET_IS_SENSITIVE (range))
+
+  if ((!range->inverted && (arrow_type == GTK_ARROW_DOWN ||
+                            arrow_type == GTK_ARROW_RIGHT)) ||
+      (range->inverted  && (arrow_type == GTK_ARROW_UP ||
+                            arrow_type == GTK_ARROW_LEFT)))
+    {
+      switch (range->layout->upper_sensitivity)
+        {
+        case GTK_SENSITIVITY_AUTO:
+          arrow_sensitive =
+            (range->adjustment->value <
+             (range->adjustment->upper - range->adjustment->page_size));
+          break;
+
+        case GTK_SENSITIVITY_ON:
+          arrow_sensitive = TRUE;
+          break;
+
+        case GTK_SENSITIVITY_OFF:
+          arrow_sensitive = FALSE;
+          break;
+        }
+    }
+  else
+    {
+      switch (range->layout->lower_sensitivity)
+        {
+        case GTK_SENSITIVITY_AUTO:
+          arrow_sensitive =
+            (range->adjustment->value > range->adjustment->lower);
+          break;
+
+        case GTK_SENSITIVITY_ON:
+          arrow_sensitive = TRUE;
+          break;
+
+        case GTK_SENSITIVITY_OFF:
+          arrow_sensitive = FALSE;
+          break;
+        }
+    }
+
+  if (!GTK_WIDGET_IS_SENSITIVE (range) || !arrow_sensitive)
     state_type = GTK_STATE_INSENSITIVE;
   else if (clicked)
     state_type = GTK_STATE_ACTIVE;
@@ -934,8 +1078,8 @@ draw_stepper (GtkRange     *range,
     state_type = GTK_STATE_PRELIGHT;
   else 
     state_type = GTK_STATE_NORMAL;
-  
-  if (clicked)
+
+  if (clicked && arrow_sensitive)
     shadow_type = GTK_SHADOW_IN;
   else
     shadow_type = GTK_SHADOW_OUT;
@@ -955,7 +1099,7 @@ draw_stepper (GtkRange     *range,
   arrow_x = widget->allocation.x + rect->x + (rect->width - arrow_width) / 2;
   arrow_y = widget->allocation.y + rect->y + (rect->height - arrow_height) / 2;
   
-  if (clicked)
+  if (clicked && arrow_sensitive)
     {
       gint arrow_displacement_x;
       gint arrow_displacement_y;
@@ -984,10 +1128,17 @@ gtk_range_expose (GtkWidget      *widget,
   GtkRange *range;
   gboolean sensitive;
   GtkStateType state;
+  GtkShadowType shadow_type;
   GdkRectangle expose_area;	/* Relative to widget->allocation */
   GdkRectangle area;
   gint focus_line_width = 0;
   gint focus_padding = 0;
+  gboolean touchscreen;
+  gboolean activate_slider;
+
+  g_object_get (gtk_widget_get_settings (widget),
+                "gtk-touchscreen-mode", &touchscreen,
+                NULL);
 
   range = GTK_RANGE (widget);
 
@@ -1041,13 +1192,26 @@ gtk_range_expose (GtkWidget      *widget,
                          range->range_rect.height);
     }
 
+  shadow_type = GTK_SHADOW_OUT;
+
   if (!sensitive)
     state = GTK_STATE_INSENSITIVE;
-  else if (range->layout->mouse_location == MOUSE_SLIDER)
+  else if (!touchscreen && range->layout->mouse_location == MOUSE_SLIDER)
     state = GTK_STATE_PRELIGHT;
   else
     state = GTK_STATE_NORMAL;
 
+  if (range->layout->grab_location == MOUSE_SLIDER)
+    {
+      gtk_widget_style_get (widget, "activate_slider", &activate_slider, NULL);
+      
+      if (activate_slider)
+        {
+          state = GTK_STATE_ACTIVE;
+          shadow_type = GTK_SHADOW_IN;
+        }
+    }
+
   if (gdk_rectangle_intersect (&expose_area,
                                &range->layout->slider,
                                &area))
@@ -1058,7 +1222,7 @@ gtk_range_expose (GtkWidget      *widget,
       gtk_paint_slider (widget->style,
                         widget->window,
                         state,
-                        GTK_SHADOW_OUT,
+                        shadow_type,
                         &area,
                         widget,
                         GTK_RANGE_GET_CLASS (range)->slider_detail,
@@ -1073,28 +1237,28 @@ gtk_range_expose (GtkWidget      *widget,
     draw_stepper (range, &range->layout->stepper_a,
                   range->orientation == GTK_ORIENTATION_VERTICAL ? GTK_ARROW_UP : GTK_ARROW_LEFT,
                   range->layout->grab_location == MOUSE_STEPPER_A,
-                  range->layout->mouse_location == MOUSE_STEPPER_A,
+                  !touchscreen && range->layout->mouse_location == MOUSE_STEPPER_A,
                   &expose_area);
 
   if (range->has_stepper_b)
     draw_stepper (range, &range->layout->stepper_b,
                   range->orientation == GTK_ORIENTATION_VERTICAL ? GTK_ARROW_DOWN : GTK_ARROW_RIGHT,
                   range->layout->grab_location == MOUSE_STEPPER_B,
-                  range->layout->mouse_location == MOUSE_STEPPER_B,
+                  !touchscreen && range->layout->mouse_location == MOUSE_STEPPER_B,
                   &expose_area);
 
   if (range->has_stepper_c)
     draw_stepper (range, &range->layout->stepper_c,
                   range->orientation == GTK_ORIENTATION_VERTICAL ? GTK_ARROW_UP : GTK_ARROW_LEFT,
                   range->layout->grab_location == MOUSE_STEPPER_C,
-                  range->layout->mouse_location == MOUSE_STEPPER_C,
+                  !touchscreen && range->layout->mouse_location == MOUSE_STEPPER_C,
                   &expose_area);
 
   if (range->has_stepper_d)
     draw_stepper (range, &range->layout->stepper_d,
                   range->orientation == GTK_ORIENTATION_VERTICAL ? GTK_ARROW_DOWN : GTK_ARROW_RIGHT,
                   range->layout->grab_location == MOUSE_STEPPER_D,
-                  range->layout->mouse_location == MOUSE_STEPPER_D,
+                  !touchscreen && range->layout->mouse_location == MOUSE_STEPPER_D,
                   &expose_area);
   
   return FALSE;
@@ -1218,6 +1382,26 @@ coord_to_value (GtkRange *range,
   return value;
 }
 
+static gboolean
+gtk_range_key_press (GtkWidget   *widget,
+		     GdkEventKey *event)
+{
+  GtkRange *range = (GtkRange *)widget;
+
+  if (event->keyval == GDK_Escape)
+    {
+      stop_scrolling (range);
+
+      update_slider_position (range, 
+			      range->slide_initial_coordinate,
+			      range->slide_initial_coordinate);
+
+      return TRUE;
+    }
+  else
+    return GTK_WIDGET_CLASS (gtk_range_parent_class)->key_press_event (widget, event);
+}
+
 static gint
 gtk_range_button_press (GtkWidget      *widget,
 			GdkEventButton *event)
@@ -1286,6 +1470,7 @@ gtk_range_button_press (GtkWidget      *widget,
            range->layout->mouse_location == MOUSE_SLIDER)
     {
       gboolean need_value_update = FALSE;
+      gboolean activate_slider;
 
       /* Any button can be used to drag the slider, but you can start
        * dragging the slider with a trough click using button 2;
@@ -1332,11 +1517,17 @@ gtk_range_button_press (GtkWidget      *widget,
           range->slide_initial_coordinate = event->x;
         }
 
+      range_grab_add (range, MOUSE_SLIDER, event->button);
+      
+      gtk_widget_style_get (widget, "activate_slider", &activate_slider, NULL);
+      
+      /* force a redraw, if the active slider is drawn differently to the prelight one */
+      if (activate_slider)
+        gtk_widget_queue_draw (widget);
+      
       if (need_value_update)
         update_slider_position (range, event->x, event->y);
 
-      range_grab_add (range, MOUSE_SLIDER, event->button);
-      
       return TRUE;
     }
   
@@ -1564,14 +1755,42 @@ gtk_range_state_changed (GtkWidget    *widget,
     stop_scrolling (GTK_RANGE (widget));
 }
 
+#define check_rectangle(rectangle1, rectangle2)              \
+  {                                                          \
+    if (rectangle1.x != rectangle2.x) return TRUE;           \
+    if (rectangle1.y != rectangle2.y) return TRUE;           \
+    if (rectangle1.width  != rectangle2.width)  return TRUE; \
+    if (rectangle1.height != rectangle2.height) return TRUE; \
+  }
+
+static gboolean
+layout_changed (GtkRangeLayout *layout1, 
+		GtkRangeLayout *layout2)
+{
+  check_rectangle (layout1->slider, layout2->slider);
+  check_rectangle (layout1->trough, layout2->trough);
+  check_rectangle (layout1->stepper_a, layout2->stepper_a);
+  check_rectangle (layout1->stepper_d, layout2->stepper_d);
+  check_rectangle (layout1->stepper_b, layout2->stepper_b);
+  check_rectangle (layout1->stepper_c, layout2->stepper_c);
+
+  return FALSE;
+}
+
 static void
 gtk_range_adjustment_changed (GtkAdjustment *adjustment,
 			      gpointer       data)
 {
   GtkRange *range = GTK_RANGE (data);
+  /* create a copy of the layout */
+  GtkRangeLayout layout = *range->layout;
 
   range->need_recalc = TRUE;
-  gtk_widget_queue_draw (GTK_WIDGET (range));
+  gtk_range_calc_layout (range, range->adjustment->value);
+  
+  /* now check whether the layout changed  */
+  if (layout_changed (range->layout, &layout))
+    gtk_widget_queue_draw (GTK_WIDGET (range));
 
   /* Note that we don't round off to range->round_digits here.
    * that's because it's really broken to change a value
@@ -1587,13 +1806,21 @@ gtk_range_adjustment_value_changed (GtkAdjustment *adjustment,
 				    gpointer       data)
 {
   GtkRange *range = GTK_RANGE (data);
+  /* create a copy of the layout */
+  GtkRangeLayout layout = *range->layout;
 
   range->need_recalc = TRUE;
-
-  gtk_widget_queue_draw (GTK_WIDGET (range));
-  /* This is so we don't lag the widget being scrolled. */
-  if (GTK_WIDGET_REALIZED (range))
-    gdk_window_process_updates (GTK_WIDGET (range)->window, FALSE);
+  gtk_range_calc_layout (range, range->adjustment->value);
+  
+  /* now check whether the layout changed  */
+  if (layout_changed (range->layout, &layout))
+    {
+      gtk_widget_queue_draw (GTK_WIDGET (range));
+      
+      /* This is so we don't lag the widget being scrolled. */
+      if (GTK_WIDGET_REALIZED (range))
+        gdk_window_process_updates (GTK_WIDGET (range)->window, FALSE);
+    }
   
   /* Note that we don't round off to range->round_digits here.
    * that's because it's really broken to change a value
@@ -1614,7 +1841,7 @@ gtk_range_style_set (GtkWidget *widget,
 
   range->need_recalc = TRUE;
 
-  (* GTK_WIDGET_CLASS (parent_class)->style_set) (widget, previous_style);
+  (* GTK_WIDGET_CLASS (gtk_range_parent_class)->style_set) (widget, previous_style);
 }
 
 static void
@@ -2490,14 +2717,18 @@ second_timeout (gpointer data)
 static gboolean
 initial_timeout (gpointer data)
 {
-  GtkRange *range;
+  GtkRange    *range;
+  GtkSettings *settings;
+  guint        timeout;
 
   GDK_THREADS_ENTER ();
+  settings = gtk_widget_get_settings (GTK_WIDGET (data));
+  g_object_get (settings, "gtk-timeout-repeat", &timeout, NULL);
+
   range = GTK_RANGE (data);
-  range->timer->timeout_id = 
-    g_timeout_add (SCROLL_LATER_DELAY,
-                   second_timeout,
-                   range);
+  range->timer->timeout_id = g_timeout_add (timeout * SCROLL_DELAY_FACTOR,
+                                            second_timeout,
+                                            range);
   GDK_THREADS_LEAVE ();
 
   /* remove self */
@@ -2508,15 +2739,20 @@ static void
 gtk_range_add_step_timer (GtkRange      *range,
                           GtkScrollType  step)
 {
+  GtkSettings *settings;
+  guint        timeout;
+
   g_return_if_fail (range->timer == NULL);
   g_return_if_fail (step != GTK_SCROLL_NONE);
-  
+
+  settings = gtk_widget_get_settings (GTK_WIDGET (range));
+  g_object_get (settings, "gtk-timeout-initial", &timeout, NULL);
+
   range->timer = g_new (GtkRangeStepTimer, 1);
 
-  range->timer->timeout_id =
-    g_timeout_add (SCROLL_INITIAL_DELAY,
-                   initial_timeout,
-                   range);
+  range->timer->timeout_id = g_timeout_add (timeout,
+                                            initial_timeout,
+                                            range);
   range->timer->step = step;
 
   gtk_range_scroll (range, range->timer->step);
