@@ -101,24 +101,13 @@ gtk_tool_button_get_type (void)
   static GtkType type = 0;
 
   if (!type)
-    {
-      static const GTypeInfo type_info =
-	{
-	  sizeof (GtkToolButtonClass),
-	  (GBaseInitFunc) NULL,
-	  (GBaseFinalizeFunc) NULL,
-	  (GClassInitFunc) gtk_tool_button_class_init,
-	  (GClassFinalizeFunc) NULL,
-	  NULL,
-	  sizeof (GtkToolButton),
-	  0, /* n_preallocs */
-	  (GInstanceInitFunc) gtk_tool_button_init,
-	};
-
-      type = g_type_register_static (GTK_TYPE_TOOL_ITEM,
-				     "GtkToolButton",
-				     &type_info, 0);
-    }
+    type = g_type_register_static_simple (GTK_TYPE_TOOL_ITEM,
+					  I_("GtkToolButton"),
+					  sizeof (GtkToolButtonClass),
+					  (GClassInitFunc) gtk_tool_button_class_init,
+					  sizeof (GtkToolButton),
+					  (GInstanceInitFunc) gtk_tool_button_init,
+					  0);
   return type;
 }
 
@@ -126,13 +115,11 @@ static void
 gtk_tool_button_class_init (GtkToolButtonClass *klass)
 {
   GObjectClass *object_class;
-  GtkWidgetClass *widget_class;
   GtkToolItemClass *tool_item_class;
   
   parent_class = g_type_class_peek_parent (klass);
   
   object_class = (GObjectClass *)klass;
-  widget_class = (GtkWidgetClass *)klass;
   tool_item_class = (GtkToolItemClass *)klass;
   
   object_class->set_property = gtk_tool_button_set_property;
@@ -242,7 +229,7 @@ gtk_tool_button_class_init (GtkToolButtonClass *klass)
  * or activated with the keyboard.
  **/
   toolbutton_signals[CLICKED] =
-    g_signal_new ("clicked",
+    g_signal_new (I_("clicked"),
 		  G_OBJECT_CLASS_TYPE (klass),
 		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkToolButtonClass, clicked),
@@ -947,8 +934,7 @@ gtk_tool_button_set_icon_widget (GtkToolButton *button,
       
       if (icon_widget)
 	{
-	  g_object_ref (icon_widget);
-	  gtk_object_sink (GTK_OBJECT (icon_widget));
+	  g_object_ref_sink (icon_widget);
 	}
 
       button->priv->icon_widget = icon_widget;
@@ -994,8 +980,7 @@ gtk_tool_button_set_label_widget (GtkToolButton *button,
       
       if (label_widget)
 	{
-	  g_object_ref (label_widget);
-	  gtk_object_sink (GTK_OBJECT (label_widget));
+	  g_object_ref_sink (label_widget);
 	}
 
       button->priv->label_widget = label_widget;
