@@ -58,7 +58,7 @@ typedef struct _GtkPrintBackendCupsClass GtkPrintBackendCupsClass;
 #define _CUPS_MAX_CHUNK_SIZE 8192
 
 #define _CUPS_MAP_ATTR_INT(attr, v, a) {if (!g_ascii_strcasecmp (attr->name, (a))) v = attr->values[0].integer;}
-#define _CUPS_MAP_ATTR_STR(attr, v, a) {if (!g_ascii_strcasecmp (attr->name, (a))) v = g_strdup (attr->values[0].string.text);}
+#define _CUPS_MAP_ATTR_STR(attr, v, a) {if (!g_ascii_strcasecmp (attr->name, (a))) v = attr->values[0].string.text;}
 
 static GType print_backend_cups_type = 0;
 
@@ -158,26 +158,23 @@ static cairo_surface_t *    cups_printer_create_cairo_surface      (GtkPrinter
 static void
 gtk_print_backend_cups_register_type (GTypeModule *module)
 {
-  if (!print_backend_cups_type)
-    {
-      static const GTypeInfo print_backend_cups_info =
-      {
-	sizeof (GtkPrintBackendCupsClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_print_backend_cups_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkPrintBackendCups),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_print_backend_cups_init
-      };
+  static const GTypeInfo print_backend_cups_info =
+  {
+    sizeof (GtkPrintBackendCupsClass),
+    NULL,		/* base_init */
+    NULL,		/* base_finalize */
+    (GClassInitFunc) gtk_print_backend_cups_class_init,
+    NULL,		/* class_finalize */
+    NULL,		/* class_data */
+    sizeof (GtkPrintBackendCups),
+    0,		/* n_preallocs */
+    (GInstanceInitFunc) gtk_print_backend_cups_init
+  };
 
-      print_backend_cups_type = g_type_module_register_type (module,
-                                                             GTK_TYPE_PRINT_BACKEND,
-						             "GtkPrintBackendCups",
-						             &print_backend_cups_info, 0);
-    }
+  print_backend_cups_type = g_type_module_register_type (module,
+                                                         GTK_TYPE_PRINT_BACKEND,
+                                                         "GtkPrintBackendCups",
+                                                         &print_backend_cups_info, 0);
 }
 
 G_MODULE_EXPORT void 
@@ -324,15 +321,14 @@ cups_print_cb (GtkPrintBackendCups *print_backend,
       if ((attr = ippFindAttribute(response, "job-id", IPP_TAG_INTEGER)) != NULL)
 	job_id = attr->values[0].integer;
 
-
-        if (job_id == 0)
-	  gtk_print_job_set_status (ps->job, GTK_PRINT_STATUS_FINISHED);
-	else
-	  {
-	    gtk_print_job_set_status (ps->job, GTK_PRINT_STATUS_PENDING);
-	    cups_begin_polling_info (print_backend, ps->job, job_id);
-	  }
-    }
+      if (!gtk_print_job_get_track_print_status (ps->job) || job_id == 0)
+	gtk_print_job_set_status (ps->job, GTK_PRINT_STATUS_FINISHED);
+      else
+	{
+	  gtk_print_job_set_status (ps->job, GTK_PRINT_STATUS_PENDING);
+	  cups_begin_polling_info (print_backend, ps->job, job_id);
+	}
+    } 
   else
     gtk_print_job_set_status (ps->job, GTK_PRINT_STATUS_FINISHED_ABORTED);
 
@@ -550,10 +546,15 @@ cups_dispatch_watch_finalize (GSource *source)
   if (dispatch->backend)
     {
       /* We need to unref this at idle time, because it might be the
-	 last reference to this module causing the code to be
-	 unloaded (including this particular function!)
-      */
-      gtk_print_backend_unref_at_idle (GTK_PRINT_BACKEND (dispatch->backend));
+       * last reference to this module causing the code to be
+       * unloaded (including this particular function!)
+       * Update: Doing this at idle caused a deadlock taking the
+       * mainloop context lock while being in a GSource callout for
+       * multithreaded apps. So, for now we just disable unloading
+       * of print backends. See _gtk_print_backend_create for the
+       * disabling.
+       */
+      g_object_unref (dispatch->backend);
       dispatch->backend = NULL;
     }
 
@@ -635,7 +636,7 @@ cups_request_printer_info_cb (GtkPrintBackendCups *backend,
 
   /* TODO: determine printer type and use correct icon */
   gtk_printer_set_icon_name (printer, "printer");
-  
+ 
   state_msg = "";
   loc = "";
   desc = "";
@@ -652,9 +653,6 @@ cups_request_printer_info_cb (GtkPrintBackendCups *backend,
       _CUPS_MAP_ATTR_INT (attr, job_count, "queued-job-count");
     }
 
-  /* if we got a member_printer_uri then this printer is part of a class
-     so use member_printer_uri, else user printer_uri */
-
   status_changed = gtk_printer_set_job_count (printer, job_count);
   
   status_changed |= gtk_printer_set_location (printer, loc);
@@ -2489,9 +2487,9 @@ cups_printer_prepare_for_print (GtkPrinter *printer,
     gtk_print_settings_set (settings, "cups-OutputOrder", "Reverse");
   print_job->reverse = FALSE;
 
-  if (gtk_print_settings_get_num_copies (settings) > 1)
+  if (gtk_print_settings_get_n_copies (settings) > 1)
     gtk_print_settings_set_int (settings, "cups-copies",
-				gtk_print_settings_get_num_copies (settings));
+				gtk_print_settings_get_n_copies (settings));
   print_job->num_copies = 1;
 
   scale = gtk_print_settings_get_scale (settings);
